"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules import CloudFormationLintRule


class ToJsonStringResolved(CloudFormationLintRule):
    id = "W1040"
    shortdesc = "Validate the values that come from a Fn::ToJsonString function"
    description = (
        "Resolve the Fn::ToJsonString and then validate the values against the schema"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-ToJsonString.html"
    tags = ["functions", "tojsonstring"]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from cfnlint.rules.jsonschema.Base import BaseJsonSchema


class Configuration(BaseJsonSchema):
    id = "E0200"
    shortdesc = "Validate parameter file configuration"
    description = (
        "Validate if a parameter file has the correct syntax "
        "for one of the supported formats"
    )
    source_url = "https://github.com/aws-cloudformation/cfn-lint"
    tags = ["base"]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from unittest.mock import MagicMock, PropertyMock, patch

import pytest

from cfnlint.config import ConfigMixIn
from cfnlint.context import ParameterSet
from cfnlint.rules import Match
from cfnlint.rules.deployment_files.Configuration import Configuration
from cfnlint.runner.deployment_file import expand_deployment_files

_filename_dev = "deployment-dev.yaml"
_filename_prod = "deployment-prod.yaml"


def mock_glob(value, recursive):
    return [value]


@pytest.mark.parametrize(
    "name,deployment_files,expected",
    [
        (
            "A standard git sync file",
            {
                _filename_dev: (
                    {
                        "template-file-path": "../a/path",
                        "parameters": {
                            "Foo": "Bar",
                        },
                        "tags": {
                            "Key": "Value",
                        },
                    },
                    [],
                ),
            },
            [
                (
                    ConfigMixIn(
                        [],
                        deployment_files=[_filename_dev],
                        parameters=[
                            ParameterSet(
                                source=_filename_dev, parameters={"Foo": "Bar"}
                            )
                        ],
                        templates=[
                            "../a/path",
                        ],
                    ),
                    [],
                )
            ],
        ),
        (
            "Multiple standard git sync file with same template path",
            {
                _filename_dev: (
                    {
                        "template-file-path": "../a/path",
                        "parameters": {
                            "Foo": "Bar",
                        },
                        "tags": {
                            "Key": "Value",
                        },
                    },
                    [],
                ),
                _filename_prod: (
                    {
                        "template-file-path": "../a/path",
                        "parameters": {
                            "Foo": "Foo",
                        },
                        "tags": {
                            "Key": "Value",
                        },
                    },
                    [],
                ),
            },
            [
                (
                    ConfigMixIn(
                        [],
                        deployment_files=[_filename_dev, _filename_prod],
                        parameters=[
                            ParameterSet(
                                source=_filename_dev, parameters={"Foo": "Bar"}
                            ),
                            ParameterSet(
                                source=_filename_prod, parameters={"Foo": "Foo"}
                            ),
                        ],
                        templates=[
                            "../a/path",
                        ],
                    ),
                    [],
                ),
            ],
        ),
        (
            "Multiple standard git sync file with a different template path",
            {
                _filename_dev: (
                    {
                        "template-file-path": "../a/path",
                        "parameters": {
                            "Foo": "Bar",
                        },
                        "tags": {
                            "Key": "Value",
                        },
                    },
                    [],
                ),
                _filename_prod: (
                    {
                        "template-file-path": "../b/path",
                        "parameters": {
                            "Foo": "Foo",
                        },
                        "tags": {
                            "Key": "Value",
                        },
                    },
                    [],
                ),
            },
            [
                (
                    ConfigMixIn(
                        [],
                        deployment_files=[_filename_dev],
                        parameters=[
                            ParameterSet(
                                source=_filename_dev, parameters={"Foo": "Bar"}
                            ),
                        ],
                        templates=[
                            "../a/path",
                        ],
                    ),
                    [],
                ),
                (
                    ConfigMixIn(
                        [],
                        deployment_files=[_filename_prod],
                        parameters=[
                            ParameterSet(
                                source=_filename_prod, parameters={"Foo": "Foo"}
                            ),
                        ],
                        templates=[
                            "../b/path",
                        ],
                    ),
                    [],
                ),
            ],
        ),
        (
            "Bad template-file-path type",
            {
                _filename_dev: (
                    {
                        "template-file-path": ["../a/path"],
                        "parameters": {
                            "Foo": "Bar",
                        },
                        "tags": {
                            "Key": "Value",
                        },
                    },
                    [],
                )
            },
            [
                (
                    None,
                    [
                        Match(
                            linenumber=1,
                            columnnumber=1,
                            linenumberend=1,
                            columnnumberend=1,
                            filename=_filename_dev,
                            message=(
                                f"Deployment file {_filename_dev!r} is not supported"
                            ),
                            rule=Configuration(),
                        )
                    ],
                )
            ],
        ),
        (
            "Bad decode",
            {
                _filename_dev: (
                    None,
                    [
                        Match(
                            linenumber=1,
                            columnnumber=1,
                            linenumberend=1,
                            columnnumberend=1,
                            filename=_filename_dev,
                            message="Random failure",
                            rule=Configuration(),
                        )
                    ],
                ),
            },
            [
                (
                    None,
                    [
                        Match(
                            linenumber=1,
                            columnnumber=1,
                            linenumberend=1,
                            columnnumberend=1,
                            filename=_filename_dev,
                            message="Random failure",
                            rule=Configuration(),
                        ),
                    ],
                ),
            ],
        ),
    ],
)
def test_runner(
    name,
    deployment_files,
    expected,
):
    decode_results = [v for _, v in deployment_files.items()]
    deployment_files = [k for k, _ in deployment_files.items()]
    with patch("glob.glob", MagicMock(side_effect=mock_glob)):
        with patch(
            "cfnlint.config.ConfigMixIn.deployment_files", new_callable=PropertyMock
        ) as mock_deployment_files:
            with patch("cfnlint.runner.deployment_file.runner.decode") as mock_decode:
                mock_deployment_files.return_value = deployment_files
                mock_decode.side_effect = decode_results
                config = ConfigMixIn([], deployment_files=deployment_files)

                deployments = list(expand_deployment_files(config))

                for deployment_file in deployment_files:
                    mock_decode.assert_any_call(deployment_file)

                assert deployments == expected, f"{name}: {deployments} != {expected}"



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any


@dataclass(frozen=True)
class DeploymentFileData:
    template_file_path: str = field()
    parameters: dict[str, Any] = field(default_factory=dict)
    tags: dict[str, str] = field(default_factory=dict)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from unittest.mock import patch

import pytest

from cfnlint.config import ConfigMixIn
from cfnlint.context import ParameterSet
from cfnlint.rules import RulesCollection
from cfnlint.runner.template import run_template_by_data


@pytest.mark.parametrize(
    "name, config, runner_results, expected_parameters",
    [
        (
            "A standard template",
            ConfigMixIn(),
            [
                iter([]),
            ],
            [],
        ),
        (
            "One set of parameters",
            ConfigMixIn(
                parameters=[ParameterSet(source=None, parameters={"Foo": "Bar"})]
            ),
            [
                iter([]),
            ],
            [ParameterSet(source=None, parameters={"Foo": "Bar"})],
        ),
        (
            "Multiple parameters",
            ConfigMixIn(
                parameters=[
                    ParameterSet(source=None, parameters={"A": "B"}),
                    ParameterSet(source=None, parameters={"C": "D"}),
                ]
            ),
            [
                iter([]),
                iter([]),
            ],
            [
                ParameterSet(
                    source=None,
                    parameters={"A": "B"},
                ),
                ParameterSet(
                    source=None,
                    parameters={"C": "D"},
                ),
            ],
        ),
    ],
)
def test_runner(
    name,
    config,
    runner_results,
    expected_parameters,
):
    with patch(
        "cfnlint.rules._rules.RulesCollection.run", side_effect=runner_results
    ) as mock_run:
        list(run_template_by_data({}, config, RulesCollection()))

        assert mock_run.call_count == 1
        call = mock_run.call_args
        assert call.kwargs["cfn"].context.parameter_sets == expected_parameters, (
            f"{name}: {call.kwargs['cfn'].context.parameter_sets} "
            f"!= {expected_parameters}"
        )



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

__all__ = ["create_deployment_from_git_sync"]

from cfnlint.runner.deployment_file.deployment_types.git_sync import (
    create_deployment_from_git_sync,
)



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import logging
import os
import sys
from typing import Any, Iterator

import cfnlint.formatters
import cfnlint.maintenance
from cfnlint.config import ConfigMixIn, configure_logging
from cfnlint.exceptions import CfnLintExitException, UnexpectedRuleException
from cfnlint.rules import Match, Rules
from cfnlint.rules.errors import ConfigError
from cfnlint.runner.deployment_file.runner import expand_deployment_files
from cfnlint.runner.parameter_file.runner import expand_parameter_files
from cfnlint.runner.template import (
    run_template_by_data,
    run_template_by_file_paths,
    run_template_by_pipe,
)
from cfnlint.schema import PROVIDER_SCHEMA_MANAGER, patch

LOGGER = logging.getLogger(__name__)


def get_formatter(config: ConfigMixIn) -> cfnlint.formatters.BaseFormatter:
    """
    Get a formatter instance based on the specified configuration.

    Args:
        config (ConfigMixIn): The configuration object containing the format setting.

    Returns:
        cfnlint.formatters.BaseFormatter: A formatter instance based
        on the specified format.

    """
    fmt = config.format
    if fmt:
        if fmt == "quiet":
            return cfnlint.formatters.QuietFormatter()
        if fmt == "parseable":
            # pylint: disable=bad-option-value
            return cfnlint.formatters.ParseableFormatter()
        if fmt == "json":
            return cfnlint.formatters.JsonFormatter()
        if fmt == "junit":
            return cfnlint.formatters.JUnitFormatter()
        if fmt == "pretty":
            return cfnlint.formatters.PrettyFormatter()
        if fmt == "sarif":
            return cfnlint.formatters.SARIFFormatter()

    return cfnlint.formatters.Formatter()


class Runner:
    """
    Runs a set of rules against one or more CloudFormation templates.

    Attributes:
        config (ConfigMixIn): The configuration object containing
        settings for the template scan.
        formatter (BaseFormatter): The formatter used to output the
        results of the template scan.
        rules (Rules): The set of rules to be applied to the templates.

    Methods:
        _get_rules() -> None:
            Load the rules to be applied to the templates.
        _validate_filenames(filenames: Sequence[str | None]) -> Iterator[Match]:
            Validate the specified filenames and yield any matches found.
        validate_template(filename: str | None,
         template: dict[str, Any]) -> Iterator[Match]:
            Validate a single CloudFormation template and yield any matches found.
        _cli_output(matches: list[Match]) -> None:
            Output the results of the template scan to the console or a file.
        _exit(matches: list[Match]) -> int:
            Determine the appropriate exit code based on the severity of the matches.
        run() -> Iterator[Match]:
            Run the template validation process and yield any matches found.
        cli() -> None:
            Run the template validation process and output the results.
    """

    def __init__(self, config: ConfigMixIn) -> None:
        """
        Initialize a new Runner instance.

        Args:
            config (ConfigMixIn): The configuration object containing
                settings for the template scan.
        """
        self.config = config
        self.formatter = get_formatter(self.config)
        self.rules: Rules = Rules()
        self._get_rules()
        try:
            self.config.templates
        except ValueError as e:
            self._cli_output([Match(str(e), ConfigError(), None)])
        # load registry schemas before patching
        if self.config.registry_schemas:
            for path in self.config.registry_schemas:
                PROVIDER_SCHEMA_MANAGER.load_registry_schemas(path)
        # now we can patch after loading all registry schemas
        if self.config.override_spec:
            patch(self.config.override_spec, self.config.regions)

    def _get_rules(self) -> None:
        """
        Load the rules to be applied to the templates.
        """
        self.rules = Rules()
        try:
            for rules_path in self.config.append_rules:
                if rules_path and os.path.isdir(os.path.expanduser(rules_path)):
                    self.rules.update(Rules.create_from_directory(rules_path))
                else:
                    self.rules.update(Rules.create_from_module(rules_path))
            self.rules.update(
                Rules.create_from_custom_rules_file(self.config.custom_rules)
            )
        except (OSError, ImportError) as e:
            raise UnexpectedRuleException(
                f"Tried to append rules but got an error: {str(e)}", 1
            ) from e

    def validate_template(self, template: dict[str, Any]) -> Iterator[Match]:
        """
        Validate a single CloudFormation template and yield any matches found.

        This function decodes the provided template, validates it against the
        configured rules, and yields any matches found as an iterator.

        Args:
            filename (str | None): The filename of the template being validated.
            template (dict[str, Any]): The CloudFormation template to be validated.

        Yields:
            Match: The matches found during the validation process.

        Raises:
            None: This function does not raise any exceptions.
        """
        yield from run_template_by_data(template, self.config, self.rules)

    def _cli_output(self, matches: list[Match]) -> None:
        formatter = get_formatter(self.config)
        matches.sort(key=lambda x: (x.filename, x.linenumber, x.rule.id))
        output = formatter.print_matches(matches, self.rules, config=self.config)
        if output:
            if self.config.output_file:
                with open(self.config.output_file, "w") as output_file:
                    output_file.write(output)
            else:
                print(output)

        self._exit(matches)

    def _exit(self, matches: list[Match]) -> int:
        """Determine exit code"""

        exit_level: str = self.config.non_zero_exit_code or "informational"

        exit_levels: dict[str, list[str]] = {
            "informational": ["informational", "warning", "error"],
            "warning": ["warning", "error"],
            "error": ["error"],
            "none": [],
        }

        exit_code = 0
        for match in matches:
            if (
                match.rule.severity == "informational"
                and match.rule.severity in exit_levels[exit_level]
            ):
                exit_code = exit_code | 8
            elif (
                match.rule.severity == "warning"
                and match.rule.severity in exit_levels[exit_level]
            ):
                exit_code = exit_code | 4
            elif (
                match.rule.severity == "error"
                and match.rule.severity in exit_levels[exit_level]
            ):
                exit_code = exit_code | 2

        sys.exit(exit_code)

    def run(self) -> Iterator[Match]:
        """
        Run the rules against the CloudFormation template and yield
        the resulting matches.

        This function is responsible for executing the validation
        process against the CloudFormation template. It performs
        the following steps:

        1. Transform the template, if necessary, and yield any matches found.
        2. Build the template graph, if configured.
        3. Run the configured rules against the template and
        yield the resulting matches.
        4. Filter the matches based on any metadata directives in the template.
        5. Deduplicate the matches before yielding them.

        Yields:
            Match: The matches found by running the rules against the template.

        Raises:
            None: This function does not raise any exceptions.
        """

        if (
            not sys.stdin.isatty()
            and not self.config.templates
            and not self.config.deployment_files
        ):
            yield from run_template_by_pipe(self.config, self.rules)
            return

        if self.config.deployment_files:
            for template_config, matches in expand_deployment_files(self.config):
                if not template_config:
                    yield from matches
                    continue
                yield from run_template_by_file_paths(template_config, self.rules)

            return

        if self.config.parameter_files:
            for parameter_config, matches in expand_parameter_files(self.config):
                if not parameter_config:
                    yield from matches
                    continue

                yield from run_template_by_file_paths(parameter_config, self.rules)

        yield from run_template_by_file_paths(self.config, self.rules)

    def cli(self) -> None:
        """
        Run the template validation process and output the results.

        This function is the entry point for the command-line interface (CLI) of the
        CloudFormation linter.

        If no templates are specified and the input is a terminal, the function will
        print the usage information and exit with a non-zero status code.

        Returns:
            None: This function does not return a value.

        Raises:
            None: This function does not raise any exceptions.
        """
        # Add our logging configuration when running CLI
        configure_logging(self.config.debug, self.config.info)

        if self.config.update_specs:
            cfnlint.maintenance.update_resource_specs(self.config.force)
            sys.exit(0)

        if self.config.patch_specs:
            cfnlint.maintenance.patch_resource_specs()
            sys.exit(0)

        if self.config.update_iam_policies:
            cfnlint.maintenance.update_iam_policies()
            sys.exit(0)

        if self.config.update_documentation:
            # Get ALL rules (ignore the CLI settings))
            cfnlint.maintenance.update_documentation(self.rules)
            sys.exit(0)

        if self.config.listrules:
            print(self.rules)
            sys.exit(0)

        if self.config.listtemplates:
            if not self.config.templates:
                print("None")
                sys.exit(0)

            # Separate directories and files
            directories = []
            files = []
            for template in self.config.templates:
                if os.path.isdir(template):
                    directories.append(f"not a file: {template}")
                else:
                    files.append(template)

            # Print directories first, then files
            for msg in directories:
                print(msg)
            for file in files:
                print(file)

            sys.exit(0)

        # Use centralized configuration validation
        # For CLI, we allow stdin input when no templates/deployment files are specified
        try:
            self.config.validate(allow_stdin=True)
        except ValueError as e:
            print(f"Configuration error: {e}")
            self.config.parser.print_help()
            sys.exit(1)

        if not self.config.templates and not self.config.deployment_files:
            if sys.stdin.isatty():
                self.config.parser.print_help()
                sys.exit(1)

        try:
            self._cli_output(list(self.run()))
        except CfnLintExitException as e:
            LOGGER.error(str(e))
            sys.exit(e.exit_code)


def main() -> None:
    try:
        config = ConfigMixIn(sys.argv[1:])
    except Exception as e:
        print(e)
        sys.exit(1)
    runner = Runner(config)
    runner.cli()


# for pyinstaller
if __name__ == "__main__":  # pragma: no cover
    main()



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import logging
from typing import Any, Iterator

from cfnlint.config import ConfigMixIn
from cfnlint.context.parameters import ParameterSet
from cfnlint.decode.decode import decode
from cfnlint.rules import Match
from cfnlint.rules.parameter_files.Configuration import Configuration

LOGGER = logging.getLogger(__name__)


def _parse_parameter_file(
    filename: str,
) -> dict[str, Any] | list[Match]:
    data, matches = decode(filename)

    if data is None:
        return matches

    key = "ParameterKey"
    value = "ParameterValue"
    parameters: dict[str, Any] = {}
    for parameter in data:
        if not all(p in parameter for p in [key, value]):
            return [
                Match(
                    linenumber=1,
                    columnnumber=1,
                    linenumberend=1,
                    columnnumberend=1,
                    filename=filename,
                    message=f"Parameter file {filename!r} is not supported",
                    rule=Configuration(),
                )
            ]

        parameters[parameter.get(key)] = parameter.get(value)  # type: ignore

    return parameters


def expand_parameter_files(
    config: ConfigMixIn,
) -> Iterator[tuple[ConfigMixIn | None, list[Match]]]:
    """
    Expands parameter files into an evolved config

    Args:
        config (ConfigMixIn): The configuration object containing
        settings for the deployment file scan.

    Yields:

    """

    parameters: list[ParameterSet] = []
    for parameter_file in config.parameter_files:
        parameter_data = _parse_parameter_file(parameter_file)

        if not isinstance(parameter_data, dict):
            yield None, parameter_data
            continue

        parameters.append(ParameterSet(parameter_file, parameter_data))

    if parameters:
        parameters_config = config.evolve(
            parameters=parameters,
            deployment_files=[],
        )
        yield parameters_config, []



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import logging
from pathlib import Path
from typing import Iterator

import cfnlint.runner.deployment_file.deployment_types
from cfnlint.config import ConfigMixIn
from cfnlint.context.parameters import ParameterSet
from cfnlint.decode import decode
from cfnlint.rules import Match, RuleMatch
from cfnlint.rules.deployment_files.Configuration import Configuration
from cfnlint.runner.deployment_file.deployment import DeploymentFileData

LOGGER = logging.getLogger(__name__)


def _parse_deployment_file(
    filename: str,
) -> DeploymentFileData | list[Match]:
    data, matches = decode(filename)

    if data is None:
        return matches

    all_matches: list[RuleMatch] = []
    for plugin in cfnlint.runner.deployment_file.deployment_types.__all__:
        deployment_data, deployment_matches = getattr(
            cfnlint.runner.deployment_file.deployment_types, plugin
        )(data)
        if deployment_matches:
            all_matches.extend(deployment_matches)
            continue

        return deployment_data  # type: ignore

    for match in all_matches:
        LOGGER.debug(
            f"While tring to process deployment file got error: {match.message}"
        )

    return [
        Match(
            linenumber=1,
            columnnumber=1,
            linenumberend=1,
            columnnumberend=1,
            filename=filename,
            message=f"Deployment file {filename!r} is not supported",
            rule=Configuration(),
        )
    ]


def expand_deployment_files(
    config: ConfigMixIn,
) -> Iterator[tuple[ConfigMixIn | None, list[Match]]]:
    """
    Run the deployment files specified in the configuration.

    Args:
        config (ConfigMixIn): The configuration object containing
        settings for the deployment file scan.

    Yields:

    """

    deployments: dict[str, list[ParameterSet]] = {}
    for deployment_file in config.deployment_files:
        deployment_data = _parse_deployment_file(deployment_file)

        if not isinstance(deployment_data, DeploymentFileData):
            yield None, deployment_data
            continue

        try:
            template_path = (
                (Path(deployment_file).parent / deployment_data.template_file_path)
                .resolve()
                .relative_to(Path.cwd())
            )
        except ValueError:
            LOGGER.debug(
                f"Template file path {deployment_data.template_file_path!r} "
                "is not relative to the current working directory"
            )
            template_path = (
                Path(deployment_file).parent / deployment_data.template_file_path
            )

        if str(template_path) in deployments:
            deployments[str(template_path)].append(
                ParameterSet(deployment_file, deployment_data.parameters)
            )
        else:
            deployments[str(template_path)] = [
                ParameterSet(deployment_file, deployment_data.parameters)
            ]

    for path, parameter_sets in deployments.items():
        template_config = config.evolve(
            templates=[path],
            parameters=parameter_sets,
            deployment_files=[],
        )
        yield template_config, []



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from pathlib import Path

import pytest

from cfnlint.config import ConfigMixIn
from cfnlint.rules import Match
from cfnlint.rules.deployment_files.Parameters import Parameters
from cfnlint.rules.functions.RefResolved import RefResolved
from cfnlint.runner import Runner


@pytest.mark.parametrize(
    "name,parameter_files,templates,expected",
    [
        (
            "Correctly configured deployment file",
            ["test/fixtures/parameter_files/prod.json"],
            ["test/fixtures/templates/integration/deployment-file-template.yaml"],
            [],
        ),
        (
            "Incorrectly configured deployment file",
            ["test/fixtures/parameter_files/dev.json"],
            ["test/fixtures/templates/integration/deployment-file-template.yaml"],
            [
                Match(
                    message="'ImageId' is a required property",
                    rule=Parameters(),
                    filename=str(Path("test/fixtures/parameter_files/dev.json")),
                    linenumber=1,
                    linenumberend=1,
                    columnnumber=1,
                    columnnumberend=11,
                ),
                Match(
                    message=(
                        "Additional properties are not allowed "
                        "('Environment' was unexpected)"
                    ),
                    rule=Parameters(),
                    filename=str(Path("test/fixtures/parameter_files/dev.json")),
                    linenumber=3,
                    linenumberend=3,
                    columnnumber=25,
                    columnnumberend=38,
                ),
            ],
        ),
        (
            "Multiple deployment files",
            [
                "test/fixtures/parameter_files/dev.json",
                "test/fixtures/parameter_files/test.json",
                "test/fixtures/parameter_files/stage.json",
                "test/fixtures/parameter_files/prod.json",
            ],
            ["test/fixtures/templates/integration/deployment-file-template.yaml"],
            [
                Match(
                    message="'ImageId' is a required property",
                    rule=Parameters(),
                    filename=str(Path("test/fixtures/parameter_files/dev.json")),
                    linenumber=1,
                    linenumberend=1,
                    columnnumber=1,
                    columnnumberend=11,
                ),
                Match(
                    message=(
                        "Additional properties are not allowed "
                        "('Environment' was unexpected)"
                    ),
                    rule=Parameters(),
                    filename=str(Path("test/fixtures/parameter_files/dev.json")),
                    linenumber=3,
                    linenumberend=3,
                    columnnumber=25,
                    columnnumberend=38,
                ),
                Match(
                    message="'host' is not one of ['default', 'dedicated']",
                    rule=Parameters(),
                    filename=str(Path("test/fixtures/parameter_files/stage.json")),
                    linenumber=11,
                    linenumberend=11,
                    columnnumber=25,
                    columnnumberend=34,
                ),
                Match(
                    message=(
                        "{'Ref': 'Affinity'} is not one of ['default', 'host'] "
                        "when 'Ref' is resolved to 'dne' from "
                        f"[{str(Path('test/fixtures/parameter_files/stage.json'))!r}, "
                        f"{str(Path('test/fixtures/parameter_files/test.json'))!r}]"
                    ),
                    rule=RefResolved(),
                    filename=str(
                        Path(
                            "test/fixtures/templates/integration/deployment-file-template.yaml"
                        )
                    ),
                    linenumber=34,
                    linenumberend=34,
                    columnnumber=7,
                    columnnumberend=15,
                ),
                Match(
                    message=(
                        "{'Ref': 'ImageId'} is not a 'AWS::EC2::Image.Id' with "
                        "pattern '^ami-([0-9a-z]{8}|[0-9a-z]{17})$' when 'Ref' "
                        "is resolved to 'ami-zxyzabc123' from "
                        f"[{str(Path('test/fixtures/parameter_files/stage.json'))!r}, "
                        f"{str(Path('test/fixtures/parameter_files/test.json'))!r}]"
                    ),
                    rule=RefResolved(),
                    filename=str(
                        Path(
                            "test/fixtures/templates/integration/deployment-file-template.yaml"
                        )
                    ),
                    linenumber=35,
                    linenumberend=35,
                    columnnumber=7,
                    columnnumberend=14,
                ),
            ],
        ),
    ],
)
def test_parameter_files(
    name,
    parameter_files,
    templates,
    expected,
):
    config = ConfigMixIn(
        cli_args=[],
        parameter_files=parameter_files,
        templates=templates,
    )

    runner = Runner(config)
    results = list(runner.run())

    assert results == expected, f"{name}: {results} != {expected}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from cfnlint._typing import Any, RuleMatches
from cfnlint.jsonschema import StandardValidator
from cfnlint.rules.jsonschema.Base import BaseJsonSchema


class Configuration(BaseJsonSchema):
    id = "E0100"
    shortdesc = "Validate deployment file configuration"
    description = (
        "Validate if a deployment file has the correct syntax "
        "for one of the supported formats"
    )
    source_url = "https://github.com/aws-cloudformation/cfn-lint"
    tags = ["base"]

    def validate_deployment_file(
        self, data: dict[str, Any], schema: dict[str, Any]
    ) -> RuleMatches:
        matches = []

        validator = StandardValidator(schema)

        matches.extend(self.json_schema_validate(validator, data, []))

        return matches
